require 'spec_helper'

describe 'puppet_enterprise::profile::console::apache_proxy' do
  let(:confdir) { '/etc/puppetlabs/httpd/conf.d' }
  let(:cert_dir) { '/opt/puppet/share/console-services/certs' }

  before :all do
    @facter_facts = {
      'osfamily'          => 'Debian',
      'operatingsystem'   => 'Debian',
      'lsbmajdistrelease' => '6',
      'puppetversion'     => '3.6.2 (Puppet Enterprise 3.3.0)',
      'is_pe'             => 'true',
      'fqdn'              => 'console.rspec',
      'clientcert'        => 'awesomecert',
      'pe_concat_basedir' => '/tmp/file',
    }

    @localcacert = '/etc/puppetlabs/puppet/ssl/certs/ca.pem'

    @params = {
      'certname' => 'console_services.rspec',
      'trapperkeeper_proxy_listen_address' => '0.0.0.0',
      'trapperkeeper_proxy_listen_port' => 1234,
      'ssl_listen_address' => '0.0.0.0',
      'ssl_listen_port' => 6789,
    }
  end

  let(:facts) { @facter_facts }
  let(:localcacert) { @localcacert }
  let(:params) { @params }
  let(:dhparam_file) { '/opt/puppet/share/puppet-dashboard/certs/dhparam_puppetproxy.pem' }
  let(:default_ssl_cert) { "/opt/puppet/share/puppet-dashboard/certs/dhparam_puppetproxy_cert.pem" }
  let(:default_ssl_priv_key) { "#{cert_dir}/console_services.rspec.private_key.pem" }

  context "when the parameters are valid" do
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[Listen\s*0\.0\.0\.0:6789]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateFile\s*#{default_ssl_cert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateKeyFile\s*#{default_ssl_priv_key}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateChainFile\s*#{localcacert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCACertificateFile\s*#{localcacert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[ProxyPass\s*/\s*http://0.0.0.0:1234]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with(:owner => "root", :group => "root", :mode => "0644") }
  end

  context "when browser_ssl_cert and browser_ssl_private_key are set" do
    before :all do
      @params = {
        'certname' => 'console_services.rspec',
        'trapperkeeper_proxy_listen_address' => '0.0.0.0',
        'trapperkeeper_proxy_listen_port' => 1234,
        'ssl_listen_address' => '0.0.0.0',
        'ssl_listen_port' => 6789,
        'browser_ssl_cert' => '/some/path/to/a/browser.cert.pem',
        'browser_ssl_private_key' => '/some/path/to/a/browser.private_key.pem',
      }
    end

    it { should contain_puppet_enterprise__console__dhparam(dhparam_file).with(:ssl_certificate_file => @params['browser_ssl_cert']) }

    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[Listen\s*0\.0\.0\.0:6789]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateFile\s*#{default_ssl_cert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateKeyFile\s*#{@params['browser_ssl_private_key']}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateChainFile\s*#{localcacert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCACertificateFile\s*#{localcacert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[ProxyPass\s*/\s*http://0.0.0.0:1234]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with(:owner => "root", :group => "root", :mode => "0644") }
  end

  context "when only browser_ssl_cert is set" do
    before :all do
      @params = {
        'certname' => 'console_services.rspec',
        'trapperkeeper_proxy_listen_address' => '0.0.0.0',
        'trapperkeeper_proxy_listen_port' => 1234,
        'ssl_listen_address' => '0.0.0.0',
        'ssl_listen_port' => 6789,
        'browser_ssl_cert' => '/some/path/to/a/browser.cert.pem',
      }
    end

    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[Listen\s*0\.0\.0\.0:6789]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateFile\s*#{default_ssl_cert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateKeyFile\s*#{default_ssl_priv_key}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateChainFile\s*#{localcacert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCACertificateFile\s*#{localcacert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[ProxyPass\s*/\s*http://0.0.0.0:1234]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with(:owner => "root", :group => "root", :mode => "0644") }
  end

  context "when only browser_ssl_private_key is set" do
    before :all do
      @params = {
        'certname' => 'console_services.rspec',
        'trapperkeeper_proxy_listen_address' => '0.0.0.0',
        'trapperkeeper_proxy_listen_port' => 1234,
        'ssl_listen_address' => '0.0.0.0',
        'ssl_listen_port' => 6789,
        'browser_ssl_private_key' => '/some/path/to/a/browser.private_key.pem',
      }
    end

    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[Listen\s*0\.0\.0\.0:6789]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateFile\s*#{default_ssl_cert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateKeyFile\s*#{default_ssl_priv_key}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateChainFile\s*#{localcacert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCACertificateFile\s*#{localcacert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[ProxyPass\s*/\s*http://0.0.0.0:1234]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with(:owner => "root", :group => "root", :mode => "0644") }
  end

  context "when browser_ssl_cert, browser_ssl_private_key and browser_ssl_cert_chain are set" do
    before :all do
      @params = {
        'certname' => 'console_services.rspec',
        'trapperkeeper_proxy_listen_address' => '0.0.0.0',
        'trapperkeeper_proxy_listen_port' => 1234,
        'ssl_listen_address' => '0.0.0.0',
        'ssl_listen_port' => 6789,
        'browser_ssl_cert' => '/some/path/to/a/browser.cert.pem',
        'browser_ssl_private_key' => '/some/path/to/a/browser.private_key.pem',
        'browser_ssl_cert_chain' => '/some/path/to/a/ca_chain.crt',
      }
    end
    it { should contain_puppet_enterprise__console__dhparam(dhparam_file).with(:ssl_certificate_file => @params['browser_ssl_cert']) }

    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[Listen\s*0\.0\.0\.0:6789]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateFile\s*#{default_ssl_cert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateKeyFile\s*#{@params['browser_ssl_private_key']}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateChainFile\s*#{@params['browser_ssl_cert_chain']}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCACertificateFile\s*#{localcacert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[ProxyPass\s*/\s*http://0.0.0.0:1234]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with(:owner => "root", :group => "root", :mode => "0644") }
  end

  context "when browser_ssl_cert, browser_ssl_private_key, browser_ssl_cert_chain and browser_ssl_ca_cert are set" do
    before :all do
      @params = {
        'certname' => 'console_services.rspec',
        'trapperkeeper_proxy_listen_address' => '0.0.0.0',
        'trapperkeeper_proxy_listen_port' => 1234,
        'ssl_listen_address' => '0.0.0.0',
        'ssl_listen_port' => 6789,
        'browser_ssl_cert' => '/some/path/to/a/browser.cert.pem',
        'browser_ssl_private_key' => '/some/path/to/a/browser.private_key.pem',
        'browser_ssl_cert_chain' => '/some/path/to/a/ca_chain.crt',
        'browser_ssl_ca_cert' => '/some/path/to/a/ca.crt',
      }
    end

    it { should contain_puppet_enterprise__console__dhparam(dhparam_file).with(:ssl_certificate_file => @params['browser_ssl_cert']) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[Listen\s*0\.0\.0\.0:6789]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateFile\s*#{default_ssl_cert}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateKeyFile\s*#{@params['browser_ssl_private_key']}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCertificateChainFile\s*#{@params['browser_ssl_cert_chain']}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[SSLCACertificateFile\s*#{@params['browser_ssl_ca_cert']}]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with_content(%r[ProxyPass\s*/\s*http://0.0.0.0:1234]) }
    it { should contain_file("#{confdir}/puppetproxy.conf").with(:owner => "root", :group => "root", :mode => "0644") }
  end
end
